import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { CommandHandler } from '@nestjs/cqrs';
import { ConfigurationMessagerPresenter } from '../../adapters/secondaries/configuration-messager.presenter';
import { ConfigurationMessager } from '../../adapters/secondaries/configuration.messager';
import { ConfigurationRepository } from '../../adapters/secondaries/configuration.repository';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';
import { CreateConfigurationCommand } from '../../commands/create-configuration.command';
import { CreateConfigurationRequest } from '../dtos/create-configuration.request';
import { Configuration } from '../entities/configuration';

@CommandHandler(CreateConfigurationCommand)
export class CreateConfigurationUseCase {
  constructor(
    private readonly _repository: ConfigurationRepository,
    private readonly _configurationMessager: ConfigurationMessager,
    private readonly _loggingMessager: LoggingMessager,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  async execute(command: CreateConfigurationCommand): Promise<Configuration> {
    const entity = this._mapper.map(
      command.createConfigurationRequest,
      CreateConfigurationRequest,
      Configuration,
    );

    try {
      const configuration = await this._repository.create(entity);
      this._configurationMessager.publish(
        'create',
        JSON.stringify(
          this._mapper.map(
            configuration,
            Configuration,
            ConfigurationMessagerPresenter,
          ),
        ),
      );
      this._loggingMessager.publish(
        'configuration.create.info',
        JSON.stringify(configuration),
      );
      return configuration;
    } catch (error) {
      let key = 'configuration.create.crit';
      if (error.message.includes('Already exists')) {
        key = 'configuration.create.warning';
      }
      this._loggingMessager.publish(
        key,
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  }
}
